@@ -11,17 +11,19 @@ package Lemonldap::NG::Portal::2F::Engines::Default;
 
 use strict;
 use Mouse;
+use MIME::Base64 qw(encode_base64);
 use JSON qw(from_json to_json);
 use POSIX qw(strftime);
 use Lemonldap::NG::Portal::Main::Constants qw(
+  PE_OK
   PE_ERROR
   PE_NOTOKEN
-  PE_OK
   PE_SENDRESPONSE
   PE_TOKENEXPIRED
+  PE_NO_SECOND_FACTORS
 );
 
-our $VERSION = '2.0.8';
+our $VERSION = '2.0.14';
 
 extends 'Lemonldap::NG::Portal::Main::Plugin';
 with 'Lemonldap::NG::Portal::Lib::OverConf';
@@ -44,6 +46,19 @@ has ott => (
     }
 );
 
+has regOtt => (
+    is      => 'rw',
+    lazy    => 1,
+    default => sub {
+        my $ott =
+          $_[0]->{p}->loadModule('Lemonldap::NG::Portal::Lib::OneTimeToken');
+        my $timeout = $_[0]->{conf}->{sfRegisterTimeout}
+          // $_[0]->{conf}->{formTimeout};
+        $ott->timeout($timeout);
+        return $ott;
+    }
+);
+
 sub init {
     my ($self) = @_;
 
@@ -176,12 +191,10 @@ sub init {
         # Registration base
         $self->addAuthRoute( '2fregisters' => '_displayRegister', ['GET'] );
         $self->addAuthRoute( '2fregisters' => 'register',         ['POST'] );
-        if ( $self->conf->{sfRequired} ) {
-            $self->addUnauthRoute(
-                '2fregisters' => 'restoreSession',
-                [ 'GET', 'POST' ]
-            );
-        }
+        $self->addUnauthRoute(
+            '2fregisters' => 'restoreSession',
+            [ 'GET', 'POST' ]
+        ) if ( $self->conf->{sfRequired} );
     }
     return 1;
 }
@@ -193,12 +206,36 @@ sub init {
 # run() is called at each authentication, just after sessionInfo populated
 sub run {
     my ( $self, $req ) = @_;
-    my $checkLogins = $req->param('checkLogins');
-    my $spoofId     = $req->param('spoofId') || '';
+    my $checkLogins   = $req->param('checkLogins');
+    my $stayconnected = $req->param('stayconnected');
+    my $spoofId       = $req->param('spoofId') || '';
     $self->logger->debug("2F checkLogins set") if ($checkLogins);
 
     # Skip 2F unless a module has been registered
-    return PE_OK unless ( @{ $self->sfModules } );
+    unless ( @{ $self->sfModules } ) {
+        if ( $self->conf->{sfOnlyUpgrade} and $req->data->{doingSfUpgrade} ) {
+            $self->logger->error(
+                    "Trying to perform 2FA session upgrade but no "
+                  . "second factor modules are configured" );
+            return PE_ERROR;
+        }
+        else {
+            return PE_OK;
+        }
+    }
+
+    # Skip 2F if authnLevel is already high enough
+    if (
+        $self->conf->{sfOnlyUpgrade}
+        and ( ( $req->pdata->{targetAuthnLevel} || 0 ) <=
+            ( $req->sessionInfo->{authenticationLevel} || 0 ) )
+      )
+    {
+        $self->logger->debug(
+                "Current authentication level satisfied target service,"
+              . " skipping 2FA" );
+        return PE_OK;
+    }
 
     # Remove expired 2F devices
     my $session = $req->sessionInfo;
@@ -212,11 +249,12 @@ sub run {
             $self->logger->error("Bad encoding in _2fDevices: $@");
             return PE_ERROR;
         }
-
         $self->logger->debug(" -> 2F Device(s) found");
-        my $now     = time();
-        my $removed = 0;
+
         $self->logger->debug("Looking for expired 2F device(s)...");
+        my $removed = 0;
+        my $name    = '';
+        my $now     = time();
         foreach my $device (@$_2fDevices) {
             my $type = lc( $device->{type} );
             $type =~ s/2f$//i;
@@ -228,16 +266,18 @@ sub run {
                 );
                 $self->userLogger->info("Remove expired $device->{type}");
                 $device->{type} = 'EXPIRED';
+                $name .= "$device->{name}; ";
                 $removed++;
             }
         }
 
         if ($removed) {
+            $name =~ s/;\s$//;
             $self->logger->debug(
 "Found $removed EXPIRED 2F device(s) => Update persistent session"
             );
             $self->userLogger->notice(
-                " -> $removed expired 2F device(s) removed");
+                " -> $removed expired 2F device(s) removed ($name)");
             @$_2fDevices =
               map { $_->{type} =~ /\bEXPIRED\b/ ? () : $_ } @$_2fDevices;
             $self->p->updatePersistentSession( $req,
@@ -252,13 +292,13 @@ sub run {
                 my $title = $self->conf->{sfRemovedNotifTitle}
                   || 'Second factor notification';
                 my $msg = $self->conf->{sfRemovedNotifMsg}
-                  || "$removed expired second factor(s) has/have been removed!";
-                $msg =~ s/_removedSF_/$removed/;
-
+                  || "$removed expired second factor(s) has/have been removed ($name)!";
+                $msg =~ s/\b_removedSF_\b/$removed/;
+                $msg =~ s/\b_nameSF_\b/$name/;
                 my $params =
                   $removed > 1
-                  ? { trspan => "expired2Fremoved, $removed" }
-                  : { trspan => "oneExpired2Fremoved" };
+                  ? { trspan => "expired2Fremoved, $removed, $name" }
+                  : { trspan => "oneExpired2Fremoved, $name" };
 
                 my $notifEngine = $self->p->loadedModules->{
                     'Lemonldap::NG::Portal::Plugins::Notifications'};
@@ -285,7 +325,7 @@ sub run {
             $self->logger->debug("2F is required...");
             $self->logger->debug(" -> Register 2F");
             $req->pdata->{sfRegToken} =
-              $self->ott->createToken( $req->sessionInfo );
+              $self->regOtt->createToken( $req->sessionInfo );
             $self->logger->debug("Just one 2F is enabled");
             $self->logger->debug(" -> Redirect to 2fregisters/");
             $req->response( [
@@ -296,7 +336,16 @@ sub run {
             return PE_SENDRESPONSE;
         }
         else {
-            return PE_OK;
+            if ( $self->conf->{sfOnlyUpgrade} and $req->data->{doingSfUpgrade} )
+            {
+
+                # cancel redirection to issuer/vhost
+                delete $req->pdata->{_url};
+                return PE_NO_SECOND_FACTORS;
+            }
+            else {
+                return PE_OK;
+            }
         }
     }
 
@@ -327,19 +376,23 @@ sub run {
         $req,
         '2fchoice',
         params => {
-            MAIN_LOGO => $self->conf->{portalMainLogo},
-            SKIN      => $self->p->getSkin($req),
-            LANGS     => $self->conf->{showLanguages},
-            TOKEN     => $token,
-            MODULES   => [
-                map { {
+            MAIN_LOGO     => $self->conf->{portalMainLogo},
+            SKIN          => $self->p->getSkin($req),
+            LANGS         => $self->conf->{showLanguages},
+            CHECKLOGINS   => $checkLogins,
+            STAYCONNECTED => $stayconnected,
+            TOKEN         => $token,
+            MSG           => $self->canUpdateSfa($req) || 'choose2f',
+            ALERT   => ( $self->canUpdateSfa($req) ? 'warning' : 'positive' ),
+            MODULES => [
+                map {
+                    {
                         CODE  => $_->prefix,
                         LOGO  => $_->logo,
                         LABEL => $_->label
                     }
                 } @am
             ],
-            CHECKLOGINS => $checkLogins
         }
     );
     $req->response($tpl);
@@ -348,8 +401,8 @@ sub run {
 
 # bool public display2fRegisters($req, $session)
 #
-# Return true if at least 1 register module is available for this user. Used
-# by Menu to display or not /2fregisters page
+# Return true if at least 1 register module is available for this user.
+# Used by Menu for displaying or not /2fregisters page
 sub display2fRegisters {
     my ( $self, $req, $session ) = @_;
     foreach my $m ( @{ $self->sfRModules } ) {
@@ -388,7 +441,7 @@ sub _choice {
             return $self->p->do(
                 $req,
                 [
-                    sub { $res }, 'controlUrl',
+                    sub { $res },  'controlUrl',
                     'buildCookie', @{ $self->p->endAuth },
                 ]
             );
@@ -416,20 +469,18 @@ sub _displayRegister {
     if ($tpl) {
         my ($m) =
           grep { $_->{m}->prefix eq $tpl } @{ $self->sfRModules };
-        unless ($m) {
-            return $self->p->sendError( $req,
-                'Inexistent register module', 400 );
-        }
-        unless ( $m->{r}->( $req, $req->userData ) ) {
-            return $self->p->sendError( $req,
-                'Registration not authorized', 403 );
-        }
+        return $self->p->sendError( $req, 'Inexistent register module', 400 )
+          unless $m;
+        return $self->p->sendError( $req, 'Registration not authorized', 403 )
+          unless $m->{r}->( $req, $req->userData );
         return $self->p->sendHtml(
             $req,
             $m->{m}->template,
             params => {
                 MAIN_LOGO => $self->conf->{portalMainLogo},
                 LANGS     => $self->conf->{showLanguages},
+                MSG       => $self->canUpdateSfa($req) || $m->{m}->welcome,
+                ALERT => ( $self->canUpdateSfa($req) ? 'warning' : 'positive' ),
             }
         );
     }
@@ -448,32 +499,36 @@ sub _displayRegister {
               };
         }
     }
-    if (
+
+    return [ 302, [ Location => $self->conf->{portal} . $am[0]->{URL} ], [] ]
+      if (
         @am == 1
         and not( $req->userData->{_2fDevices}
             or $req->data->{sfRegRequired} )
-      )
-    {
-        return [ 302, [ Location => $self->conf->{portal} . $am[0]->{URL} ],
-            [] ];
-    }
+      );
 
     # Retrieve user all second factors
-    my $_2fDevices =
-      $req->userData->{_2fDevices}
-      ? eval {
-        from_json( $req->userData->{_2fDevices}, { allow_nonref => 1 } ); }
-      : undef;
-    unless ($_2fDevices) {
-        $self->logger->debug("No 2F Device found");
-        $_2fDevices = [];
+    my $_2fDevices = [];
+    unless ( $self->canUpdateSfa($req) ) {
+        $_2fDevices = $req->userData->{_2fDevices}
+          ? eval {
+            from_json( $req->userData->{_2fDevices}, { allow_nonref => 1 } );
+          }
+          : undef;
+        unless ($_2fDevices) {
+            $self->logger->debug("No 2F Device found");
+            $_2fDevices = [];
+        }
+    }
+    else {
+        $self->userLogger->warn("Do not diplay 2F Devices!");
     }
 
-    # Parse second factors to display delete button if allowed
-    my $action = '';
+   # Parse second factors to display delete button if allowed and upgrade button
+    my $displayUpgBtn = 0;
+    my $action        = '';
     foreach
       my $type ( split /,\s*/, $self->conf->{available2FSelfRegistration} )
-
     {
         foreach (@$_2fDevices) {
             $_->{type} =~ s/^UBK$/Yubikey/;
@@ -481,15 +536,22 @@ sub _displayRegister {
                 my $t = lc($type);
                 $t =~ s/2f$//i;
 
+                # Display delete button
                 $_->{delAllowed} =
                      $self->conf->{ $t . '2fActivation' }
                   && $self->conf->{ $t . '2fUserCanRemoveKey' }
                   && $self->conf->{ $t . '2fSelfRegistration' };
+
+                # Display upgrade button
+                $displayUpgBtn ||= $self->conf->{ $t . '2fAuthnLevel' }
+                  && $self->conf->{ $t . '2fAuthnLevel' } >
+                  $req->userData->{authenticationLevel};
             }
             $action ||= $_->{delAllowed};
             $_->{type} =~ s/^Yubikey$/UBK/;
         }
     }
+    $displayUpgBtn = 0 unless $self->conf->{upgradeSession};
 
     # Display template
     return $self->p->sendHtml(
@@ -503,6 +565,11 @@ sub _displayRegister {
             SFDEVICES    => $_2fDevices,
             ACTION       => $action,
             REG_REQUIRED => $req->data->{sfRegRequired},
+            DISPLAY_UPG  => $displayUpgBtn,
+            MSG          => $self->canUpdateSfa($req) || 'choose2f',
+            ALERT => ( $self->canUpdateSfa($req) ? 'warning' : 'positive' ),
+            SFREGISTERS_URL =>
+              encode_base64( "$self->{conf}->{portal}2fregisters", '' )
         }
     );
 }
@@ -549,7 +616,7 @@ sub restoreSession {
     my ( $self, $req, @path ) = @_;
     my $token = $req->pdata->{sfRegToken}
       or return [ 302, [ Location => $self->conf->{portal} ], [] ];
-    $req->userData( $self->ott->getToken( $token, 1 ) );
+    $req->userData( $self->regOtt->getToken( $token, 1 ) );
     $req->data->{sfRegRequired} = 1;
     return $req->method eq 'POST'
       ? $self->register( $req, @path )
