@@ -12,7 +12,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_SENDRESPONSE
 );
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.12';
 
 extends 'Lemonldap::NG::Portal::Main::Auth', 'Lemonldap::NG::Portal::Lib::CAS';
 
@@ -72,6 +72,7 @@ sub init {
           or $a->{val} cmp $b->{val}
       } @list;
     $self->srvList( \@list );
+
     return 1;
 }
 
@@ -136,8 +137,7 @@ sub extractFormInfo {
             my $portalPath = $self->conf->{portal};
             $portalPath =~ s#^https?://[^/]+/?#/#;
 
-            $req->data->{list}            = $self->srvList;
-            $req->data->{confirmRemember} = 0;
+            $req->data->{list} = $self->srvList;
 
             $req->data->{login} = 1;
             return PE_IDPCHOICE;
@@ -248,7 +248,7 @@ sub extractFormInfo {
 }
 
 sub authenticate {
-    PE_OK;
+    return PE_OK;
 }
 
 # Set authenticationLevel.
@@ -256,7 +256,7 @@ sub setAuthSessionInfo {
     my ( $self, $req ) = @_;
     $req->{sessionInfo}->{authenticationLevel} = $self->conf->{casAuthnLevel};
     $req->{sessionInfo}->{_casSrv}             = $req->data->{_casSrvCurrent};
-    PE_OK;
+    return PE_OK;
 }
 
 sub authLogout {
@@ -272,7 +272,7 @@ sub authLogout {
     # Register CAS logout URL in logoutServices
     $req->data->{logoutServices}->{CASserver} = $logout_url;
 
-    PE_OK;
+    return PE_OK;
 }
 
 sub getDisplayType {
