@@ -8,21 +8,23 @@ package Lemonldap::NG::Portal::Auth::_WebForm;
 use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants qw(
+  PE_OK
+  PE_NOTOKEN
+  PE_FORMEMPTY
+  PE_FIRSTACCESS
   PE_CAPTCHAEMPTY
   PE_CAPTCHAERROR
-  PE_FIRSTACCESS
-  PE_FORMEMPTY
-  PE_NOTOKEN
-  PE_OK
-  PE_PASSWORDFORMEMPTY
   PE_TOKENEXPIRED
   PE_MALFORMEDUSER
+  PE_PASSWORDFORMEMPTY
 );
 
-our $VERSION = '2.0.8';
+our $VERSION = '2.0.14';
 
-extends 'Lemonldap::NG::Portal::Main::Auth',
-  'Lemonldap::NG::Portal::Lib::_tokenRule';
+extends qw(
+  Lemonldap::NG::Portal::Main::Auth
+  Lemonldap::NG::Portal::Lib::_tokenRule
+);
 
 has authnLevel => (
     is      => 'rw',
@@ -71,7 +73,10 @@ sub extractFormInfo {
     my $res            = PE_OK;
 
     # 1. No user defined at all -> first access
-    unless ( $defUser and $req->method =~ /^POST$/i ) {
+    # _pwdCheck is a workaround to make CheckUser work while using a GET
+    unless ( $defUser
+        and ( uc( $req->method ) eq "POST" or $req->data->{_pwdCheck} ) )
+    {
         $res = PE_FIRSTACCESS;
     }
 
@@ -85,11 +90,13 @@ sub extractFormInfo {
     # 3. If user and oldpassword defined -> password form
     elsif ( $defUser and $defOldPassword ) {
         $res = PE_PASSWORDFORMEMPTY
-          unless ( ( $req->{user} = $req->param('user') )
+          unless (
+               ( $req->{user} = $req->param('user') )
             && ( $req->data->{oldpassword} = $req->param('oldpassword') )
             && ( $req->data->{newpassword} = $req->param('newpassword') )
             && ( $req->data->{confirmpassword} =
-                $req->param('confirmpassword') ) );
+                $req->param('confirmpassword') )
+          );
     }
 
     # If form seems empty
@@ -138,7 +145,7 @@ sub extractFormInfo {
     # Other parameters
     $req->data->{timezone} = $req->param('timezone');
 
-    PE_OK;
+    return PE_OK;
 }
 
 # Set password in session data if wanted.
@@ -158,7 +165,7 @@ sub setAuthSessionInfo {
     # Store user timezone
     $req->{sessionInfo}->{'_timezone'} = $self->{'timezone'};
 
-    PE_OK;
+    return PE_OK;
 }
 
 # @return display type
@@ -168,6 +175,7 @@ sub getDisplayType {
 
 sub setSecurity {
     my ( $self, $req ) = @_;
+    return if $req->data->{skipToken};
 
     # If captcha is enable, prepare it
     if ( $self->captcha ) {
