@@ -2,9 +2,13 @@ package Lemonldap::NG::Portal::Issuer::SAML;
 
 use strict;
 use Mouse;
+use URI;
+use URI::QueryParam;
+use HTML::Entities qw(encode_entities);
 use Lemonldap::NG::Portal::Lib::SAML;
 use Lemonldap::NG::Portal::Main::Constants qw(
   PE_OK
+  PE_REDIRECT
   PE_SESSIONEXPIRED
   PE_SAML_ART_ERROR
   PE_SAML_DESTINATION_ERROR
@@ -18,7 +22,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_UNAUTHORIZEDPARTNER
 );
 
-our $VERSION = '2.0.9';
+our $VERSION = '2.0.14';
 
 extends 'Lemonldap::NG::Portal::Main::Issuer',
   'Lemonldap::NG::Portal::Lib::SAML';
@@ -158,6 +162,10 @@ qr/^($saml_sso_get_url|$saml_sso_get_url_ret|$saml_sso_post_url|$saml_sso_post_u
         $self->path => { relaySingleLogoutPOST => 'sloRelayPost' },
         [ 'GET', 'POST' ]
     );
+    $self->addUnauthRoute(
+        $self->path => { singleLogoutResume => 'sloResume' },
+        [ 'GET', 'POST' ]
+    );
     $self->addUnauthRoute(
         $self->path => { relaySingleLogoutTermination => 'sloRelayTerm' },
         [ 'GET', 'POST' ]
@@ -193,6 +201,10 @@ sub storeEnv {
             $req->pdata->{targetAuthnLevel} = $targetAuthnLevel
               if $targetAuthnLevel;
         }
+        if ( $login->request ) {
+            my $acs = $login->request->AssertionConsumerServiceURL;
+            $req->env->{llng_saml_acs} = $acs if $acs;
+        }
     }
     return PE_OK;
 }
@@ -236,6 +248,7 @@ sub run {
     my $idp_initiated           = $req->param('IDPInitiated');
     my $idp_initiated_sp        = $req->param('sp');
     my $idp_initiated_spConfKey = $req->param('spConfKey');
+    my $idp_initiated_spDest    = $req->param('spDest');
 
     # Normalize URL to be tolerant to SAML Path
     $url = $self->normalize_url( $url, $self->conf->{issuerDBSAMLPath},
@@ -268,7 +281,7 @@ sub run {
             $req->data->{_proxiedRequest}     = $request;
             $req->data->{_proxiedMethod}      = $method;
             $req->data->{_proxiedRelayState}  = $relaystate,
-              $req->data->{_proxiedArtifact}  = $artifact;
+              $req->data->{_proxiedArtifact} = $artifact;
         }
 
         # Process the request or use IDP initiated mode
@@ -356,12 +369,19 @@ sub run {
                   ->{$idp_initiated_spConfKey}
                   ->{samlSPMetaDataOptionsNameIDFormat} || "email";
                 eval {
+
                     $login->request()->NameIDPolicy()
                       ->Format( $self->getNameIDFormat($nameIDFormatKey) );
                 };
 
                 # Force AllowCreate to TRUE
                 eval { $login->request()->NameIDPolicy()->AllowCreate(1); };
+
+                # Allow selection the AssertionConsumerServiceURL by the user
+                if ($idp_initiated_spDest) {
+                    $login->request->AssertionConsumerServiceURL(
+                        $idp_initiated_spDest);
+                }
             }
 
             # Process authentication request
@@ -378,6 +398,10 @@ sub run {
                 return PE_SAML_SSO_ERROR;
             }
 
+            my $h =
+              $self->p->processHook( $req, 'samlGotAuthnRequest', $login );
+            return $h if ( $h != PE_OK );
+
             # Get SP entityID
             my $sp = $request ? $login->remote_providerID() : $idp_initiated_sp;
 
@@ -396,6 +420,15 @@ sub run {
             $self->logger->debug("$sp match $spConfKey SP in configuration");
             $req->env->{llng_saml_spconfkey} = $spConfKey;
 
+            if ( $login->request ) {
+                my $acs = $login->request->AssertionConsumerServiceURL;
+                if ($acs) {
+                    $req->env->{llng_saml_acs} = $acs;
+                    $self->logger->debug(
+                        "Using AssertionConsumerServiceURL $acs");
+                }
+            }
+
             # Check access rule
             if ( my $rule = $self->spRules->{$spConfKey} ) {
                 unless ( $rule->( $req, $req->sessionInfo ) ) {
@@ -427,7 +460,9 @@ sub run {
                 }
 
                 unless ($result) {
-                    $self->logger->error("Signature is not valid");
+                    $self->logger->error(
+"Could not verify signature of incoming SSO request from $spConfKey"
+                    );
                     return PE_SAML_SIGNATURE_ERROR;
                 }
                 else {
@@ -438,11 +473,35 @@ sub run {
                 $self->logger->debug("Message signature will not be checked");
             }
 
+            my $nameIDFormat;
+
+            # Check NameID Policy in request
+            if ( $login->request()->NameIDPolicy ) {
+                $nameIDFormat = $login->request()->NameIDPolicy->Format();
+                $self->logger->debug(
+                    "Get NameID format $nameIDFormat from request");
+            }
+
+            # Get default NameID Format from configuration
+            # Set to "email" if no value in configuration
+            my $nameIDFormatKey =
+              $self->conf->{samlSPMetaDataOptions}->{$spConfKey}
+              ->{samlSPMetaDataOptionsNameIDFormat} || "email";
+
+            # NameID unspecified is forced to default NameID format
+            if (  !$nameIDFormat
+                or $nameIDFormat eq $self->getNameIDFormat("unspecified") )
+            {
+                $nameIDFormat = $self->getNameIDFormat($nameIDFormatKey);
+                eval {
+                    $login->request()->NameIDPolicy()->Format($nameIDFormat);
+                };
+            }
+
             # Force AllowCreate to TRUE for transient/persistent NameIDPolicy
             if ( $login->request()->NameIDPolicy ) {
-                my $nif = $login->request()->NameIDPolicy->Format();
-                if (   $nif eq $self->getNameIDFormat("transient")
-                    or $nif eq $self->getNameIDFormat("persistent") )
+                if (   $nameIDFormat eq $self->getNameIDFormat("transient")
+                    or $nameIDFormat eq $self->getNameIDFormat("persistent") )
                 {
                     $self->logger->debug(
                         "Force AllowCreate flag in NameIDPolicy");
@@ -535,32 +594,11 @@ sub run {
 
             $self->logger->debug("SSO: assertion is built");
 
-            # Get default NameID Format from configuration
-            # Set to "email" if no value in configuration
-            my $nameIDFormatKey =
-              $self->conf->{samlSPMetaDataOptions}->{$spConfKey}
-              ->{samlSPMetaDataOptionsNameIDFormat} || "email";
-            my $nameIDFormat;
-
-            # Check NameID Policy in request
-            if ( $login->request()->NameIDPolicy ) {
-                $nameIDFormat = $login->request()->NameIDPolicy->Format();
-                $self->logger->debug(
-                    "Get NameID format $nameIDFormat from request");
-            }
-
-            # NameID unspecified is forced to default NameID format
-            if (  !$nameIDFormat
-                or $nameIDFormat eq $self->getNameIDFormat("unspecified") )
-            {
-                $nameIDFormat = $self->getNameIDFormat($nameIDFormatKey);
-            }
-
             # Get session key associated with NameIDFormat
             # Not for unspecified, transient, persistent, entity, encrypted
             my $nameIDFormatConfiguration = {
-                $self->getNameIDFormat("email") => 'samlNameIDFormatMapEmail',
-                $self->getNameIDFormat("x509")  => 'samlNameIDFormatMapX509',
+                $self->getNameIDFormat("email")   => 'samlNameIDFormatMapEmail',
+                $self->getNameIDFormat("x509")    => 'samlNameIDFormatMapX509',
                 $self->getNameIDFormat("windows") =>
                   'samlNameIDFormatMapWindows',
                 $self->getNameIDFormat("kerberos") =>
@@ -568,7 +606,9 @@ sub run {
             };
 
             my $nameIDSessionKey =
-              $self->conf->{ $nameIDFormatConfiguration->{$nameIDFormat} };
+                $nameIDFormatConfiguration->{$nameIDFormat}
+              ? $self->conf->{ $nameIDFormatConfiguration->{$nameIDFormat} }
+              : '';
 
             # Override default NameID Mapping
             if ( $self->conf->{samlSPMetaDataOptions}->{$spConfKey}
@@ -580,12 +620,16 @@ sub run {
             }
 
             my $nameIDContent;
-            if ( $self->spMacros->{$sp}->{$nameIDSessionKey} ) {
+            if (    $nameIDSessionKey
+                and $self->spMacros->{$sp}->{$nameIDSessionKey} )
+            {
                 $nameIDContent =
                   $self->spMacros->{$sp}->{$nameIDSessionKey}
                   ->( $req, $req->{sessionInfo} );
             }
-            elsif ( defined $req->{sessionInfo}->{$nameIDSessionKey} ) {
+            elsif ( $nameIDSessionKey
+                and ( defined $req->{sessionInfo}->{$nameIDSessionKey} ) )
+            {
                 $nameIDContent =
                   $self->p->getFirstValue(
                     $req->{sessionInfo}->{$nameIDSessionKey} );
@@ -619,8 +663,8 @@ sub run {
 
             $self->logger->debug(
                 "NameID Format is " . $login->nameIdentifier->Format );
-            $self->logger->debug(
-                "NameID Content is " . $login->nameIdentifier->content );
+            $self->logger->debug( "NameID Content is "
+                  . ( $login->nameIdentifier->content || "" ) );
 
             # Push attributes
             my @attributes;
@@ -656,20 +700,21 @@ sub run {
                 unless ( defined $value ) {
                     if ($mandatory) {
                         $self->logger->error(
-"Session key $_ is required to set SAML $name attribute"
+"Session key $_ is required to set SAML $name attribute ($sp)"
                         );
                         return PE_ISSUERMISSINGREQATTR;
                     }
                     else {
                         $self->logger->debug(
-"SAML2 attribute $name has no value but is not mandatory, skip it"
+"SAML2 attribute $name has no value but is not mandatory ($sp), skip it"
                         );
                         next;
                     }
                 }
 
                 $self->logger->debug(
-                    "SAML2 attribute $name will be set with $_ session key");
+"SAML2 attribute $name will be set with $_ session key ($sp)"
+                );
 
                 # SAML2 attribute
                 my $attribute =
@@ -840,6 +885,10 @@ sub run {
 "SAML authentication response sent to SAML SP $spConfKey for $user$nameIDLog"
             );
 
+            $h =
+              $self->p->processHook( $req, 'samlBuildAuthnResponse', $login );
+            return $h if ( $h != PE_OK );
+
             # Build SAML response
             $protocolProfile = $login->protocolProfile();
 
@@ -1000,8 +1049,11 @@ sub run {
                 }
 
                 # RelayState
-                $req->{postFields}->{'RelayState'} = $relaystate
-                  if ($relaystate);
+                if ($relaystate) {
+                    $req->{postFields}->{'RelayState'} =
+                      encode_entities($relaystate);
+                    $req->data->{safeHiddenFormValues}->{RelayState} = 1;
+                }
 
                 $req->steps( ['autoPost'] );
                 return PE_OK;
@@ -1117,6 +1169,12 @@ sub soapSloServer {
 
         $self->logger->debug("SLO: Logout request is valid");
 
+        my $h = $self->p->processHook( $req, 'samlGotLogoutRequest', $logout );
+        if ( $h != PE_OK ) {
+            return $self->p->sendError( $req,
+                "SLO: samlGotLogoutRequest hook returned error", 400 );
+        }
+
         # We accept only SOAP here
         unless ( $method eq $self->getHttpMethod('soap') ) {
             return $self->p->sendError( $req,
@@ -1276,9 +1334,15 @@ sub soapSloServer {
                 "SLO response signature according to metadata");
         }
 
+        $h = $self->p->processHook( $req, 'samlBuildLogoutResponse', $logout );
+        if ( $h != PE_OK ) {
+            return $self->p->sendError( $req,
+                "SLO: samlBuildLogoutResponse hook returned error", 400 );
+        }
+
         # Send logout response
         unless ( $self->buildLogoutResponseMsg($logout) ) {
-            $self->logger->error("Unable to build SLO response");
+            $self->logger->error("Unable to build SLO response for $spConfKey");
             return $self->p->sendError( $req, 'Unable to build SLO response',
                 400 );
         }
@@ -1414,7 +1478,8 @@ sub sloRelaySoap {
 
     # Store success status for this SLO request
     my $sloStatusSessionInfos =
-      $self->getSamlSession( $relayState, { $spConfKey => 1 } );
+      $self->getSamlSession( $relayState,
+        { $spConfKey => 1, _utime => time() } );
 
     if ($sloStatusSessionInfos) {
         $self->logger->debug(
@@ -1454,9 +1519,15 @@ sub sloRelayPost {
     $self->logger->debug("Found relay session $relayID");
 
     # Get data to build POST form
-    $req->{postUrl}                     = $relayInfos->data->{url};
+    $req->{postUrl} = $relayInfos->data->{url};
     $req->{postFields}->{'SAMLRequest'} = $relayInfos->data->{body};
-    $req->{postFields}->{'RelayState'}  = $relayInfos->data->{relayState};
+
+    # RelayState
+    if ( $relayInfos->data->{relayState} ) {
+        $req->{postFields}->{'RelayState'} =
+          encode_entities( $relayInfos->data->{relayState} );
+        $req->data->{safeHiddenFormValues}->{RelayState} = 1;
+    }
 
     # Delete relay session
     $relayInfos->remove();
@@ -1576,6 +1647,69 @@ sub authSloServer {
     return $self->sloServer($req);
 }
 
+sub sloResume {
+    my ( $self, $req ) = @_;
+
+    my $ResumeParams = $req->params('ResumeParams');
+
+    unless ($ResumeParams) {
+        $self->logger->error("Could not find resumption info");
+        return PE_SAML_SLO_ERROR;
+    }
+
+    my $logoutContextSession = $self->getSamlSession($ResumeParams);
+
+    unless ($logoutContextSession) {
+        $self->logger->error("Could not find logout context session");
+        return PE_SAML_SLO_ERROR;
+    }
+
+    my $spConfKey   = $logoutContextSession->data->{spConfKey};
+    my $method      = $logoutContextSession->data->{method};
+    my $provider_nb = $logoutContextSession->data->{provider_nb};
+    my $relayID     = $logoutContextSession->data->{relayID};
+
+    # Restore Lasso logout object from XML dump
+    my $logout = $self->createLogout( $self->lassoServer,
+        $logoutContextSession->data->{logout} );
+
+    # Restore session info (for logout of other SPs)
+    $req->setInfo( $logoutContextSession->data->{info} )
+      if $logoutContextSession->data->{info};
+
+    return $self->_finishSlo( $req, $logout, $method, $spConfKey,
+        $provider_nb, $relayID );
+}
+
+sub _finishSlo {
+    my ( $self, $req, $logout, $method, $spConfKey, $provider_nb, $relayID ) =
+      @_;
+
+    # Signature
+    my $signSLOMessage = $self->conf->{samlSPMetaDataOptions}->{$spConfKey}
+      ->{samlSPMetaDataOptionsSignSLOMessage};
+
+    unless ($signSLOMessage) {
+        $self->logger->debug("Do not sign this SLO response");
+        return $self->sendSLOErrorResponse( $req, $logout, $method )
+          unless ( $self->disableSignature($logout) );
+    }
+
+    # If no waiting SP, return directly SLO response
+    unless ($provider_nb) {
+        return $self->sendLogoutResponseToServiceProvider( $req, $logout,
+            $method );
+    }
+
+    # Else build SLO status relay URL and display info
+    else {
+        $req->{urldc} =
+          $self->conf->{portal} . '/saml/relaySingleLogoutTermination';
+        $self->p->setHiddenFormValue( $req, 'relay', $relayID, '', 0 );
+        return $self->p->do( $req, [] );
+    }
+}
+
 sub sloServer {
     my ( $self, $req ) = @_;
     my $url            = $req->uri;
@@ -1608,6 +1742,12 @@ sub sloServer {
 
         $self->logger->debug("SLO: Logout request is valid");
 
+        my $h = $self->p->processHook( $req, 'samlGotLogoutRequest', $logout );
+        if ( $h != PE_OK ) {
+            return $self->p->sendError( $req,
+                "SLO: samlGotLogoutRequest hook returned error", 400 );
+        }
+
         # Get SP entityID
         my $sp = $logout->remote_providerID();
         $req->env->{llng_saml_sp} = $sp;
@@ -1757,33 +1897,64 @@ sub sloServer {
         # This flag is for logout() to say that SAML logout is already done
         $req->data->{samlSLOCalled} = 1;
 
+        # This variable decides if we call the authLogout step
+        # (which can redirect away)
+        my $doAuthLogout = 0;
+        my $logoutContextSession;
+
+        # TODO: for now, we only try authLogout to disconnect an
+        # external IDP if the current session has been opened on a
+        # SAML IDP. We only propagate logout to the IDP our SP is SAML
+        # AND our IDP is SAML too
+        if ( $req->sessionInfo->{_lassoSessionDump} ) {
+
+            $doAuthLogout = 1;
+
+            # In case we have to redirect to the IDP, save current state
+            # to allow resumption. This needs to be done here because
+            # issuerUrldc will be put in the IdP logout's RelayState
+            my $logoutInfos = {
+                logout      => $logout->dump,
+                spConfKey   => $spConfKey,
+                method      => $method,
+                provider_nb => $provider_nb,
+                relayID     => $relayID,
+                _utime      => time(),
+            };
+            $logoutContextSession =
+              $self->getSamlSession( undef, $logoutInfos );
+            my $uri =
+              URI->new( $self->conf->{portal} . '/saml/singleLogoutResume' );
+            $uri->query_param( ResumeParams => $logoutContextSession->id );
+            $req->{issuerUrldc} = $uri->as_string;
+        }
+
+        # We don't want info to interfere with the auth logout process
+        my $savedInfo = $req->info;
+        $req->setInfo('');
+
         # Launch normal logout and ignore errors
-        $req->steps( [ @{ $self->p->beforeLogout }, 'deleteSession' ] );
-        $self->p->process($req);
+        $req->steps( [
+                @{ $self->p->beforeLogout },
+                ( $doAuthLogout ? 'authLogout' : () ),
+                'deleteSession'
+            ]
+        );
+        my $res = $self->p->process($req);
 
-        # Signature
-        my $signSLOMessage = $self->conf->{samlSPMetaDataOptions}->{$spConfKey}
-          ->{samlSPMetaDataOptionsSignSLOMessage};
+        if ( $res eq PE_REDIRECT ) {
 
-        unless ($signSLOMessage) {
-            $self->logger->debug("Do not sign this SLO response");
-            return $self->sendSLOErrorResponse( $req, $logout, $method )
-              unless ( $self->disableSignature($logout) );
+            # Save session info (for logout of other SP)
+            if ($savedInfo) {
+                $logoutContextSession->update( { info => $savedInfo } );
+            }
+            return $self->p->do( $req, [ sub { PE_REDIRECT } ] );
         }
 
-        # If no waiting SP, return directly SLO response
-        unless ($provider_nb) {
-            return $self->sendLogoutResponseToServiceProvider( $req, $logout,
-                $method );
-        }
+        $req->info($savedInfo);
 
-        # Else build SLO status relay URL and display info
-        else {
-            $req->{urldc} =
-              $self->conf->{portal} . '/saml/relaySingleLogoutTermination';
-            $self->p->setHiddenFormValue( $req, 'relay', $relayID, '', 0 );
-            return $self->p->do( $req, [] );
-        }
+        return $self->_finishSlo( $req, $logout, $method, $spConfKey,
+            $provider_nb, $relayID );
     }
 
     elsif ($response) {
@@ -1798,6 +1969,9 @@ sub sloServer {
 
         $self->logger->debug("Logout response is valid");
 
+        my $h = $self->p->processHook( $req, 'samlGotLogoutResponse', $logout );
+        $self->imgnok($req) if ( $h != PE_OK );
+
         # Check Destination
         $self->imgnok($req)
           unless ( $self->checkDestination( $logout->response, $url ) );
@@ -1824,7 +1998,9 @@ sub sloServer {
 
         if ($checkSLOMessageSignature) {
             unless ( $self->checkSignatureStatus($logout) ) {
-                $self->logger->error("Signature is not valid");
+                $self->logger->error(
+"Could not verify signature of incoming SLO request from $spConfKey"
+                );
                 $self->imgnok($req);
             }
             else {
@@ -1915,8 +2091,8 @@ sub attributeServer {
     my $name_id = $query->nameIdentifier();
 
     unless ($name_id) {
-        $self->p->sendError( $req, "Fail to get NameID from attribute request",
-            400 );
+        $self->p->sendError( $req,
+            "Fail to get NameID from attribute request", 400 );
     }
 
     my $user = $name_id->content();
@@ -1974,8 +2150,8 @@ sub attributeServer {
     eval { @requested_attributes = $query->request()->Attribute(); };
     if ($@) {
         $self->checkLassoError($@);
-        return $self->p->sendError( $req, "Unable to get requested attributes",
-            400 );
+        return $self->p->sendError( $req,
+            "Unable to get requested attributes", 400 );
     }
 
     # Returned attributes
