@@ -7,20 +7,14 @@ use Net::LDAP;    #inherits
 use Net::LDAP::Util qw(escape_filter_value);
 use base qw(Net::LDAP);
 use Lemonldap::NG::Portal::Main::Constants ':all';
+use Net::LDAP::Control::PasswordPolicy;
 use Encode;
 use Unicode::String qw(utf8);
 use Scalar::Util 'weaken';
+use IO::Socket::Timeout;
 use utf8;
 
-our $VERSION  = '2.0.9';
-our $ppLoaded = 0;
-
-BEGIN {
-    eval {
-        require threads::shared;
-        threads::shared::share($ppLoaded);
-    };
-}
+our $VERSION = '2.0.14';
 
 # INITIALIZATION
 
@@ -47,7 +41,8 @@ sub new {
     }
     $self = Net::LDAP->new(
         \@servers,
-        onerror => undef,
+        onerror   => undef,
+        keepalive => 1,
         ( $conf->{ldapPort}    ? ( port    => $conf->{ldapPort} )    : () ),
         ( $conf->{ldapTimeout} ? ( timeout => $conf->{ldapTimeout} ) : () ),
         ( $conf->{ldapVersion} ? ( version => $conf->{ldapVersion} ) : () ),
@@ -57,7 +52,7 @@ sub new {
         ( $conf->{ldapVerify}  ? ( verify  => $conf->{ldapVerify} )  : () ),
     );
     unless ($self) {
-        $portal->logger->error($@);
+        $portal->logger->error( "LDAP initialization error: " . $@ );
         return 0;
     }
     elsif ( $Net::LDAP::VERSION < '0.64' ) {
@@ -66,15 +61,22 @@ sub new {
         # socket when certificate validation fails. Net::LDAP does not catch
         # it, and the process ends up crashing.
         # As a precaution, make sure the underlying socket is doing fine:
-        if ( $self->socket->isa('IO::Socket::SSL')
+        if (    $self->socket->isa('IO::Socket::SSL')
             and $self->socket->errstr < 0 )
         {
             $portal->logger->error(
-                "SSL connection error: " . $self->socket->errstr );
+                "LDAP SSL connection failed: " . $self->socket->errstr );
             return 0;
         }
     }
     bless $self, $class;
+
+    # Set socket timeouts
+    my $socket = $self->socket;
+    IO::Socket::Timeout->enable_timeouts_on($socket);
+    $socket->read_timeout( $conf->{ldapIOTimeout} );
+    $socket->write_timeout( $conf->{ldapIOTimeout} );
+
     if ($useTls) {
         my %h = split( /[&=]/, $tlsParam );
         $h{cafile} ||= $conf->{ldapCAFile} if ( $conf->{ldapCAFile} );
@@ -82,7 +84,7 @@ sub new {
         $h{verify} ||= $conf->{ldapVerify} if ( $conf->{ldapVerify} );
         my $mesg = $self->start_tls(%h);
         if ( $mesg->code ) {
-            $portal->logger->error( 'StartTLS failed: ' . $mesg->error );
+            $portal->logger->error( 'LDAP StartTLS failed: ' . $mesg->error );
             return 0;
         }
     }
@@ -126,7 +128,43 @@ sub bind {
             };
             print STDERR "$@\n" if ($@);
         }
+
+        if ( $self->{conf}->{ldapPpolicyControl} ) {
+            my $pp = Net::LDAP::Control::PasswordPolicy->new();
+            $args{control} = [$pp];
+        }
+
         $mesg = $self->SUPER::bind( $dn, %args );
+
+        if ( $mesg->code ) {
+            my ($resp) = $mesg->control("1.3.6.1.4.1.42.2.27.8.5.1");
+
+            # Check for ppolicy error
+            my $pp_error = $resp->pp_error if ( defined($resp) );
+            if ( defined $pp_error ) {
+                my $ppolicy_error = [
+                    "password expired",
+                    "account locked",
+                    "change after reset",
+                    "password mod not allowed",
+                    "supply old password",
+                    "insufficient password quality",
+                    "password too short",
+                    "password too young",
+                    "password in history"
+                ]->[$pp_error];
+
+                $self->{portal}
+                  ->logger->error( "Error when binding to LDAP server: "
+                      . $mesg->error
+                      . " | extended ppolicy control response error: $ppolicy_error"
+                  );
+            }
+            else {
+                $self->{portal}->logger->error(
+                    "Error when binding to LDAP server: " . $mesg->error );
+            }
+        }
     }
     else {
         $mesg = $self->SUPER::bind();
@@ -149,30 +187,6 @@ sub unbind {
     return $mesg;
 }
 
-## @method private boolean loadPP ()
-# Load Net::LDAP::Control::PasswordPolicy
-# @return true if succeed.
-sub loadPP {
-    my $self = shift;
-    return 1 if ($ppLoaded);
-
-    # Minimal version of Net::LDAP required
-    if ( $Net::LDAP::VERSION < 0.38 ) {
-        die(
-"Module Net::LDAP is too old for password policy, please install version 0.38 or higher"
-        );
-    }
-
-    # Require Perl module
-    eval { require Net::LDAP::Control::PasswordPolicy };
-    if ($@) {
-        $self->{portal}->logger->error(
-            "Module Net::LDAP::Control::PasswordPolicy not found in @INC");
-        return 0;
-    }
-    $ppLoaded = 1;
-}
-
 ## @method protected int userBind(string dn, hash args)
 # Call bind() with dn/password and return
 # @param $dn LDAP distinguish name
@@ -193,7 +207,7 @@ sub userBind {
         # Get server control response
         my ($resp) = $mesg->control("1.3.6.1.4.1.42.2.27.8.5.1");
 
-        # Return direct unless control resonse
+        # Return direct unless control response
         unless ( defined $resp ) {
             if ( $mesg->code == 49 ) {
                 $self->{portal}->userLogger->warn(
@@ -446,7 +460,7 @@ sub userModifyPassword {
             return PE_LDAPERROR;
         }
 
-        $self->{portal}->userLogger->notice("Password changed for $dn");
+        $self->{portal}->logger->notice("Password changed for $dn");
 
         # Rebind as manager for next LDAP operations if we were bound as user
         $self->bind() if $asUser;
@@ -579,8 +593,7 @@ sub userModifyPassword {
         return PE_WRONGMANAGERACCOUNT
           if ( $mesg->code == 50 || $mesg->code == 8 );
         if ( $mesg->code == 0 ) {
-            $self->{portal}
-              ->userLogger->notice("Password changed $self->{portal}->{user}");
+            $self->{portal}->logger->notice("Password changed for $dn");
 
            # Rebind as manager for next LDAP operations if we were bound as user
             $self->bind() if $asUser;
@@ -615,39 +628,6 @@ sub userModifyPassword {
     }
 }
 
-## @method protected Lemonldap::NG::Portal::_LDAP ldap()
-# @return Lemonldap::NG::Portal::_LDAP object
-sub ldap {
-    my $self = shift;
-    return $self->{ldap}
-      if ( ref( $self->{ldap} )
-        and $self->{flags}->{ldapActive} );
-    if ( $self->{ldap} = Lemonldap::NG::Portal::_LDAP->new($self)
-        and my $mesg = $self->{ldap}->bind )
-    {
-        if ( $mesg->code != 0 ) {
-            $self->logger->error( "LDAP error: " . $mesg->error );
-            $self->{ldap}->unbind;
-        }
-        else {
-            if ( $self->{ldapPpolicyControl}
-                and not $self->{ldap}->loadPP() )
-            {
-                $self->logger->error("LDAP password policy error");
-                $self->{ldap}->unbind;
-            }
-            else {
-                $self->{flags}->{ldapActive} = 1;
-                return $self->{ldap};
-            }
-        }
-    }
-    else {
-        $self->logger->error("LDAP error: $@");
-    }
-    return 0;
-}
-
 ## @method string searchGroups(string base, string key, string value, string attributes, hashref dupcheck)
 # Get groups from LDAP directory
 # @param base LDAP search base
