@@ -6,7 +6,7 @@ use XML::LibXML;
 use XML::LibXSLT;
 use POSIX qw(strftime);
 
-our $VERSION = '2.0.8';
+our $VERSION = '2.0.12';
 
 # Lemonldap::NG::Portal::Main::Plugin provides addAuthRoute() and
 # addUnauthRoute() methods in addition of Lemonldap::NG::Common::Module.
@@ -52,7 +52,7 @@ has server   => ( is => 'rw' );
 # INITIALIZATION
 
 sub init {
-    1;
+    return 1;
 }
 
 # Search for notifications and if any, returns HTML fragment.
@@ -66,10 +66,10 @@ sub checkForNotifications {
     unless ($notifs) {
         $self->logger->info("No notification found");
         return 0;
-    };
+    }
 
     # Transform notifications
-    my $i   = 0;                                # Files count
+    my $i   = 0;    # Files count
     my $now = strftime "%Y-%m-%d", localtime;
 
     foreach my $file ( values %$notifs ) {
@@ -173,7 +173,7 @@ sub viewNotification {
     unless ($notifs) {
         $self->logger->info("No accepted notification found");
         return 0;
-    };
+    }
 
     # Transform notifications
     my $i = 0;    # Files count
@@ -247,7 +247,7 @@ sub getNotifBack {
                 name    => $self->conf->{cookieName},
                 value   => 0,
                 domain  => $self->conf->{domain},
-                secure  => 0,
+                secure  => $self->conf->{securedCookie},
                 expires => 'Wed, 21 Oct 2015 00:00:00 GMT'
             )
         );
