@@ -1,6 +1,6 @@
 package Lemonldap::NG::Portal::Main::Process;
 
-our $VERSION = '2.0.10';
+our $VERSION = '2.0.12';
 
 package Lemonldap::NG::Portal::Main;
 
@@ -57,6 +57,10 @@ sub processHook {
             $self->logger->debug("Not a code ref: $sub");
         }
     }
+    if ( $err != PE_OK ) {
+        $self->logger->warn(
+            "Hook $hookName returned " . portalConsts->{$err} );
+    }
     return $err;
 }
 
@@ -152,12 +156,16 @@ sub controlUrl {
         }
 
         # Unprotected hosts
-        my ( $proto, $vhost, $appuri ) = $tmp =~ m#^(https?://)([^/]*)(.*)#;
-        $vhost =~ s/:\d+$//;
+        if ( $tmp and ( $tmp !~ URIRE ) ) {
+            $self->userLogger->error("Bad URL $tmp");
+            delete $req->{urldc};
+            return PE_BADURL;
+        }
+        my ( $proto, $vhost, $appuri ) = ( $2, $3, $5 );
 
         # Try to resolve alias
         my $originalVhost = $self->HANDLER->resolveAlias($vhost);
-        $vhost = $proto . $originalVhost;
+        $vhost = $proto . '://' . $originalVhost;
         $self->logger->debug( "Required URL (param: "
               . ( $req->param('logout') ? 'HTTP Referer' : 'urldc' )
               . " | value: $tmp | alias: $vhost)" );
@@ -328,7 +336,7 @@ sub checkXSSAttack {
 
     # Test value
     $value =~ s/\%25/\%/g;
-    if ( $value =~ m/(?:\0|<|'|"|`|\%(?:00|3C|22|27|2C))/ ) {
+    if ( $value =~ m/(?:\0|<|'|"|`|\%(?:00|3C|22|27))/ ) {
         $self->userLogger->error(
             "XSS attack detected (param: $name | value: $value)");
         return $self->conf->{checkXSS};
@@ -373,6 +381,12 @@ sub getUser {
     return $self->_userDB->getUser( $req, %args );
 }
 
+sub findUser {
+    my ( $self, $req, %args ) = @_;
+    return PE_ERROR unless ( $self->_userDB );
+    return $self->_userDB->findUser( $req, %args );
+}
+
 sub authenticate {
     my ( $self, $req ) = @_;
     my $ret = $req->authResult( $self->_authentication->authenticate($req) );
@@ -465,13 +479,12 @@ sub setGroups {
 }
 
 sub setPersistentSessionInfo {
-
-    # $user passed by BruteForceProtection plugin
-    my ( $self, $req, $user ) = @_;
+    my ( $self, $req ) = @_;
 
     # Do not restore infos if session already opened
     unless ( $req->id ) {
-        my $key = $req->{sessionInfo}->{ $self->conf->{whatToTrace} } || $user;
+        my $key = $req->{sessionInfo}->{ $self->conf->{whatToTrace} };
+
         return PE_OK unless ( $key and length($key) );
 
         my $persistentSession = $self->getPersistentSession($key);
@@ -492,6 +505,8 @@ sub setPersistentSessionInfo {
 
 sub setLocalGroups {
     my ( $self, $req ) = @_;
+    $req->{sessionInfo}->{groups}  //= '';
+    $req->{sessionInfo}->{hGroups} //= {};
     foreach ( sort keys %{ $self->_groups } ) {
         if ( $self->_groups->{$_}->( $req, $req->sessionInfo ) ) {
             $req->{sessionInfo}->{groups} .=
@@ -610,9 +625,9 @@ sub secondFactor {
 }
 
 sub storeHistory {
-    my ( $self, $req, $uid ) = @_;  # $uid passed by BruteForceProtection plugin
+    my ( $self, $req ) = @_;
     if ( $self->conf->{loginHistoryEnabled} ) {
-        $self->registerLogin( $req, $uid );
+        $self->registerLogin($req);
     }
     PE_OK;
 }
