@@ -9,7 +9,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_BADCREDENTIALS
 );
 
-our $VERSION = '2.0.12';
+our $VERSION = '2.0.14';
 
 extends qw(
   Lemonldap::NG::Portal::Main::Plugin
@@ -28,11 +28,14 @@ has ott => (
         return $ott;
     }
 );
+
+has displayHistoryRule           => ( is => 'rw', default => sub { 0 } );
 has unrestrictedUsersRule        => ( is => 'rw', default => sub { 0 } );
 has displayEmptyValuesRule       => ( is => 'rw', default => sub { 0 } );
 has displayEmptyHeadersRule      => ( is => 'rw', default => sub { 0 } );
 has displayPersistentInfoRule    => ( is => 'rw', default => sub { 0 } );
 has displayComputedSessionRule   => ( is => 'rw', default => sub { 0 } );
+has displayHiddenAttributesRule  => ( is => 'rw', default => sub { 0 } );
 has displayNormalizedHeadersRule => ( is => 'rw', default => sub { 0 } );
 has idRule                       => ( is => 'rw', default => sub { 1 } );
 has sorted                       => ( is => 'rw', default => sub { 0 } );
@@ -97,6 +100,7 @@ sub init {
         )
     );
     return 0 unless $self->displayComputedSessionRule;
+
     $self->displayNormalizedHeadersRule(
         $self->p->buildRule(
             $self->conf->{checkUserDisplayNormalizedHeaders},
@@ -105,6 +109,22 @@ sub init {
     );
     return 0 unless $self->displayNormalizedHeadersRule;
 
+    $self->displayHistoryRule(
+        $self->p->buildRule(
+            $self->conf->{checkUserDisplayHistory},
+            'checkUserDisplayHistory'
+        )
+    );
+    return 0 unless $self->displayHistoryRule;
+
+    $self->displayHiddenAttributesRule(
+        $self->p->buildRule(
+            $self->conf->{checkUserDisplayHiddenAttributes},
+            'checkUserDisplayHiddenAttributes'
+        )
+    );
+    return 0 unless $self->displayHistoryRule;
+
     # Init. other options
     $self->sorted( $self->conf->{impersonationRule}
           || $self->conf->{contextSwitchingRule} );
@@ -116,13 +136,18 @@ sub init {
 
 # RUNNING METHODS
 sub display {
-    my ( $self,  $req )         = @_;
+    my ( $self, $req ) = @_;
+    my $history = [ [], [] ];
     my ( $attrs, $array_attrs ) = ( $req->userData, [] );
 
     $self->logger->debug("Display current session data...");
     $self->userLogger->info("Using spoofed SSO groups if exist")
       if ( $self->conf->{impersonationRule} );
 
+    $history = $self->_concatHistory( $attrs->{_loginHistory} )
+      if $self->displayHistoryRule->( $req, $req->userData )
+      && $self->conf->{loginHistoryEnabled};
+
     $attrs =
       $self->_removeKeys( $attrs, $self->persistentAttrs,
         'Remove persistent session attributes...' )
@@ -141,6 +166,9 @@ sub display {
         MSG        => 'checkUser' . $self->merged,
         ALERTE     => ( $self->merged ? 'alert-warning' : 'alert-info' ),
         LOGIN      => $req->{userData}->{ $self->conf->{whatToTrace} },
+        HISTORY    => ( @{ $history->[0] } || @{ $history->[1] } ) ? 1 : 0,
+        SUCCESS    => $history->[0],
+        FAILED     => $history->[1],
         ATTRIBUTES => $array_attrs->[2],
         MACROS     => $array_attrs->[1],
         GROUPS     => $array_attrs->[0],
@@ -161,7 +189,8 @@ sub check {
     my ( $attrs, $array_attrs, $array_hdrs ) = ( {}, [], [] );
     my $msg           = my $auth = my $computed = '';
     my $savedUserData = $req->userData;
-    my $unUser = $self->unrestrictedUsersRule->( $req, $savedUserData ) || 0;
+    my $unUser  = $self->unrestrictedUsersRule->( $req, $savedUserData ) || 0;
+    my $history = [ [], [] ];
 
     # Check token
     if ( $self->ottRule->( $req, {} ) ) {
@@ -250,7 +279,7 @@ sub check {
           . $self->conf->{checkUserSearchAttributes}
           : $self->conf->{whatToTrace};
 
-        foreach ( split /\s+/, $searchAttrs ) {
+        foreach ( split /[,\s]+/, $searchAttrs ) {
             $self->logger->debug("Searching with: $_ = $user");
             $sessions = $self->module->searchOn( $moduleOptions, $_, $user );
             last if ( keys %$sessions );
@@ -294,7 +323,11 @@ sub check {
         $attrs       = {};
     }
     else {
-        $msg = 'checkUser' . $self->merged;
+        $msg     = 'checkUser' . $self->merged;
+        $history = $self->_concatHistory( $attrs->{_loginHistory} )
+          if $self->displayHistoryRule->( $req, $savedUserData )
+          && $self->conf->{loginHistoryEnabled};
+
         $attrs =
           $self->_removeKeys( $attrs, $self->persistentAttrs,
             'Remove persistent session attributes...' )
@@ -387,6 +420,9 @@ sub check {
         ALLOWED     => $auth,
         ALERTE_AUTH => $alert_auth,
         HEADERS     => $array_hdrs,
+        HISTORY     => ( @{ $history->[0] } || @{ $history->[1] } ) ? 1 : 0,
+        SUCCESS     => $history->[0],
+        FAILED      => $history->[1],
         ATTRIBUTES  => $array_attrs->[2],
         MACROS      => $array_attrs->[1],
         GROUPS      => $array_attrs->[0],
@@ -555,23 +591,16 @@ sub _createArray {
     my ( $self, $req, $attrs, $userData ) = @_;
     my $array_attrs = [];
 
-    if ( $self->displayEmptyValuesRule->( $req, $userData ) ) {
-        $self->logger->debug("Delete hidden attributes...");
-        foreach my $k ( sort keys %$attrs ) {
-
-            # Ignore hidden attributes
-            push @$array_attrs, { key => $k, value => $attrs->{$k} }
-              unless ( $self->hAttr =~ /\b$k\b/ );
-        }
-    }
-    else {
-        $self->logger->debug("Delete hidden and empty attributes...");
-        foreach my $k ( sort keys %$attrs ) {
-
-            # Ignore hidden attributes and empty values
-            push @$array_attrs, { key => $k, value => $attrs->{$k} }
-              unless ( $self->hAttr =~ /\b$k\b/ or !$attrs->{$k} );
-        }
+    foreach my $k ( sort keys %$attrs ) {
+        push @$array_attrs,
+          { key => $k, value => $attrs->{$k} }
+          unless ( (
+                $self->hAttr =~ /\b$k\b/
+                && !$self->displayHiddenAttributesRule->( $req, $userData )
+            )
+            || (   !$attrs->{$k}
+                && !$self->displayEmptyValuesRule->( $req, $userData ) )
+          );
     }
 
     return $array_attrs;
@@ -651,4 +680,32 @@ sub _removeKeys {
     return $attrs;
 }
 
+sub _concatHistory {
+    my ( $self,    $history ) = @_;
+    my ( $success, $failed )  = ( [], [] );
+
+    $self->logger->debug('Concatenate history...');
+    @$success = map {
+        my $element = $_;
+        my $utime   = delete $element->{_utime};
+        {
+            utime  => $utime,
+            values => join $self->conf->{multiValuesSeparator},
+            map "$_=$element->{$_}", sort keys %$element
+        }
+    } @{ $history->{successLogin} };
+
+    @$failed = map {
+        my $element = $_;
+        my $utime   = delete $element->{_utime};
+        {
+            utime  => $utime,
+            values => join $self->conf->{multiValuesSeparator},
+            map "$_=$element->{$_}", sort keys %$element
+        }
+    } @{ $history->{failedLogin} };
+
+    return [ $success, $failed ];
+}
+
 1;
