@@ -40,7 +40,7 @@ has unrestrictedUsersRule => ( is => 'rw', default => sub { 0 } );
 
 sub init {
     my ($self) = @_;
-    $self->addAuthRoute( switchcontext => 'run',  ['POST'] )
+    $self->addAuthRoute( switchcontext => 'run', ['POST'] )
       ->addAuthRoute( switchcontext => 'display', ['GET'] );
 
     # Parse ContextSwitching rules
@@ -119,15 +119,14 @@ sub display {
 
     # Display form
     my $params = {
-        PORTAL    => $self->conf->{portal},
-        MAIN_LOGO => $self->conf->{portalMainLogo},
-        SKIN      => $self->p->getSkin($req),
-        LANGS     => $self->conf->{showLanguages},
-        MSG       => 'contextSwitching_ON',
-        ALERTE    => 'alert-danger',
-        LOGIN     => '',
-        SPOOFID   => $self->conf->{contextSwitchingRule},
-        TOKEN     => (
+        PORTAL        => $self->conf->{portal},
+        MAIN_LOGO     => $self->conf->{portalMainLogo},
+        SKIN          => $self->p->getSkin($req),
+        LANGS         => $self->conf->{showLanguages},
+        MSG           => 'contextSwitching_ON',
+        ALERTE        => 'alert-danger',
+        IMPERSONATION => $self->conf->{contextSwitchingRule},
+        TOKEN         => (
               $self->ottRule->( $req, {} )
             ? $self->ott->createToken()
             : ''
@@ -142,7 +141,7 @@ sub run {
     my $statut  = PE_OK;
     my $realId  = $req->userData->{ $self->conf->{whatToTrace} };
     my $spoofId = $req->param('spoofId') || '';    # ContextSwitching required ?
-    my $unUser = $self->unrestrictedUsersRule->( $req, $req->userData ) || 0;
+    my $unUser  = $self->unrestrictedUsersRule->( $req, $req->userData ) || 0;
 
     # Check token
     if ( $self->ottRule->( $req, {} ) ) {
