@@ -6,26 +6,30 @@ use Lemonldap::NG::Portal::Main::Constants;
 
 extends 'Lemonldap::NG::Common::Module';
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.12';
 
 # INITIALIZATION
 
 sub init {
-    1;
+    return 1;
 }
 
 # RUNNING METHODS
 
 sub getUser {
-    PE_OK;
+    return PE_OK;
+}
+
+sub findUser {
+    return PE_OK;
 }
 
 sub setSessionInfo {
-    PE_OK;
+    return PE_OK;
 }
 
 sub setGroups {
-    PE_OK;
+    return PE_OK;
 }
 
 1;
