@@ -5,11 +5,6 @@ extends 'Lemonldap::NG::Portal::Main::Plugin';
 
 use constant hook => { samlGotAuthnRequest => 'gotRequest', };
 
-sub init {
-    my ($self) = @_;
-    return 1;
-}
-
 sub gotRequest {
     my ( $self, $res, $login ) = @_;
 
